package cz.simecek.inspectionmanager.controller;

import cz.simecek.inspectionmanager.dto.InspectionDTO;
import cz.simecek.inspectionmanager.service.inspection.InspectionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.validation.Valid;

@Controller
public class InspectionController {

    private final InspectionService inspectionService;

    private static final String REDIRECT_TO_INDEX = "redirect:/all/inspection";
    private static final String REDIRECT_TO_DELETED = "redirect:/renew/inspection";

    private static final String INDEX_TEMPLATE = "/inspection/list";
    private static final String DETAIL_TEMPLATE = "/inspection/detail";
    private static final String UPDATE_TEMPLATE = "/inspection/update";
    private static final String CREATE_TEMPLATE = "/inspection/new";
    private static final String RENEW_TEMPLATE = "/inspection/renew";
    private static final String HISTORY_TEMPLATE = "/history/history";

    private static final String INSPECTIONS = "inspections";
    private static final String INSPECTION_DTO = "inspectionDTO";
    private static final String HISTORIES = "histories";
    private static final String MACHINE_ID = "machineId";
    private static final String HEADER = "header";
    private static final String SUCCESS_MESSAGE = "successMessage";

    @Autowired
    public InspectionController(InspectionService inspectionService) {
        this.inspectionService = inspectionService;
    }

    @GetMapping("/all/inspection")
    public String index(Model model) {
        model.addAttribute(INSPECTIONS, inspectionService.getAll());
        return INDEX_TEMPLATE;
    }

    @GetMapping("/renew/inspection")
    public String getDeleted(Model model) {
        model.addAttribute(INSPECTIONS, inspectionService.getDeleted());
        return RENEW_TEMPLATE;
    }

    @PostMapping("/renew/inspection/{id}")
    public String renewCustomer(@PathVariable Long id, RedirectAttributes redirectAttributes) {
        inspectionService.renew(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kontrola byla úspěšně obnovena.");
        return REDIRECT_TO_DELETED;
    }

    @GetMapping("/filter/inspection")
    public String filter(@RequestParam(name = MACHINE_ID) Long machineId, Model model) {
        model.addAttribute(INSPECTIONS, inspectionService.getByMachine(machineId));
        model.addAttribute(MACHINE_ID, machineId);
        return INDEX_TEMPLATE;
    }

    @GetMapping("/new/inspection")
    public String createForm(@RequestParam(name = MACHINE_ID) Long machineId, Model model) {
        model.addAttribute(MACHINE_ID, machineId);
        model.addAttribute(INSPECTION_DTO, new InspectionDTO());
        return CREATE_TEMPLATE;
    }

    @PostMapping("/new/inspection")
    public String create(@RequestParam(name = MACHINE_ID) Long machineId, @Valid InspectionDTO inspectionDTO,
                         BindingResult result, Model model, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            model.addAttribute(MACHINE_ID, machineId);
            return CREATE_TEMPLATE;
        }
        inspectionService.save(inspectionDTO, machineId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kontrola byla úspěšně uložena.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/detail/inspection/{id}")
    public String detail(@PathVariable("id") Long inspectionId, Model model) {
        model.addAttribute(INSPECTION_DTO, inspectionService.getById(inspectionId));
        return DETAIL_TEMPLATE;
    }

    @GetMapping("/update/inspection/{id}")
    public String updateForm(@PathVariable("id") Long inspectionId, Model model) {
        model.addAttribute(INSPECTION_DTO, inspectionService.getById(inspectionId));
        return UPDATE_TEMPLATE;
    }

    @PostMapping("update/inspection/{id}")
    public String update(@PathVariable("id") Long inspectionId, @Valid InspectionDTO inspectionDTO,
                         BindingResult result, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            inspectionDTO.setId(inspectionId);
            return UPDATE_TEMPLATE;
        }
        inspectionService.update(inspectionDTO, inspectionId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kontrola byla úspěšně aktualizována.");
        return REDIRECT_TO_INDEX;
    }

    @PostMapping("/delete/inspection/{id}")
    public String delete(@PathVariable("id") Long inspectionId, RedirectAttributes redirectAttributes) {
        inspectionService.delete(inspectionId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kontrola byla úspěšně smazána.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/history/inspection/{id}")
    public String getHistory(@PathVariable Long id, Model model) {
        model.addAttribute(HISTORIES, inspectionService.getHistoryById(id));
        model.addAttribute(HEADER, "kontroly");
        return HISTORY_TEMPLATE;
    }
}
