package cz.simecek.inspectionmanager.controller;

import cz.simecek.inspectionmanager.dto.SparePartDTO;
import cz.simecek.inspectionmanager.service.spare_part.SparePartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.validation.Valid;
import java.util.List;

@Controller
public class SparePartController {

    private SparePartService sparePartService;

    private final String REDIRECT_PART_DETAIL = "redirect:/detail/part/%s";
    private final String REDIRECT_TO_MACHINE_PARTS_LIST = "redirect:/detail/part/machine/%s";
    private static final String REDIRECT_TO_DELETED = "redirect:/renew/machine";

    private final String LIST_BY_MACHINE_TEMPLATE = "/parts/list";
    private final String CREATE_TEMPLATE = "/parts/new";
    private final String PART_DETAIL_TEMPLATE = "/parts/detail";
    private final String PART_UPDATE_TEMPLATE = "/parts/update";
    private static final String HISTORY_TEMPLATE = "/history/history";
    private static final String RENEW_TEMPLATE = "/parts/renew";

    private final String SPARE_PARTS = "spareParts";
    private final String SPARE_PART = "sparePart";
    private final String MACHINE_ID = "machineId";
    private final String SPARE_PART_ID = "sparePartId";
    private final String SPARE_PART_DTO = "sparePartDTO";
    private final String HISTORIES = "histories";
    private final String HEADER = "header";
    private static final String SUCCESS_MESSAGE = "successMessage";

    @Autowired
    public SparePartController(SparePartService sparePartService) {
        this.sparePartService = sparePartService;
    }

    @GetMapping("/detail/part/machine/{id}")
    public String getAllByMachineId(@PathVariable Long id, Model model) {
        List<SparePartDTO> dtoList = sparePartService.getByMachineId(id);
        model.addAttribute(MACHINE_ID, id);
        model.addAttribute(SPARE_PARTS, dtoList);
        return LIST_BY_MACHINE_TEMPLATE;
    }

    @GetMapping("/renew/part")
    public String getDeletedParts(Model model) {
        model.addAttribute(SPARE_PARTS, sparePartService.getDeleted());
        return RENEW_TEMPLATE;
    }

    @PostMapping("/renew/part/{id}")
    public String renewPart(@PathVariable Long id, Model model, RedirectAttributes redirectAttributes) {
        sparePartService.renew(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Náhradní díl byl úspěšně obnoven.");
        return REDIRECT_TO_DELETED;
    }

    @GetMapping("/new/part/machine/{id}")
    public String createFormToMachineId(@PathVariable Long id, Model model) {
        model.addAttribute(SPARE_PART_DTO, new SparePartDTO());
        model.addAttribute(MACHINE_ID, id);
        return CREATE_TEMPLATE;
    }

    @PostMapping("/new/part/machine/{id}")
    public String createPartByMachineId(@PathVariable Long id, @Valid SparePartDTO sparePartDTO, BindingResult result, Model model, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            model.addAttribute(MACHINE_ID, id);
            return CREATE_TEMPLATE;
        }
        sparePartService.save(sparePartDTO, id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Náhradní díl byl úspěšně vytvořen.");
        return String.format(REDIRECT_TO_MACHINE_PARTS_LIST, id);
    }

    @GetMapping("/detail/part/{id}")
    public String getDetail(@PathVariable Long id, Model model) {
        model.addAttribute(sparePartService.getById(id));
        return PART_DETAIL_TEMPLATE;
    }

    @PostMapping("/delete/part/{id}")
    public String deletePart(@PathVariable Long id, RedirectAttributes redirectAttributes) {
        sparePartService.delete(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Náhradní díl byl úspěšně smazán.");
        return REDIRECT_TO_DELETED;
    }

    @GetMapping("/update/part/{id}")
    public String updateForm(@PathVariable Long id, Model model) {
        model.addAttribute(SPARE_PART_ID, id);
        model.addAttribute(SPARE_PART, sparePartService.getById(id));
        return PART_UPDATE_TEMPLATE;
    }

    @PostMapping("/update/part/{id}")
    public String updatePart(@PathVariable Long id, @Valid SparePartDTO dto, BindingResult result, Model model, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            model.addAttribute(SPARE_PART_ID, id);
            return PART_UPDATE_TEMPLATE;
        }
        sparePartService.update(dto, id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Náhradní díl byl úspěšně aktualizován.");
        return String.format(REDIRECT_PART_DETAIL, id);
    }

    @GetMapping("/history/part/{id}")
    public String getHistory(@PathVariable Long id, Model model) {
        model.addAttribute(HISTORIES, sparePartService.getHistoryById(id));
        model.addAttribute(HEADER, "náhradního dílu");
        return HISTORY_TEMPLATE;
    }
}
