package cz.simecek.inspectionmanager.exception;

import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.method.annotation.MethodArgumentTypeMismatchException;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

@ControllerAdvice
public class GlobalExceptionHandler {

    private static final String ERROR_TEMPLATE = "/error/error";
    private static final String USER_PASSWORD_TEMPLATE = "/user/user_password";
    private static final String MESSAGE = "message";
    private static final String FAILURE_MESSAGE = "failureMessage";

    @ExceptionHandler(PropertyNotFoundException.class)
    public String handleNotFoundProperty(PropertyNotFoundException exception, Model model) {
        model.addAttribute(MESSAGE, exception.getMessage());
        return ERROR_TEMPLATE;
    }

    @ExceptionHandler(UnsafeDeleteException.class)
    public String handleUnsafeDelete(UnsafeDeleteException exception, Model model) {
        model.addAttribute(MESSAGE, exception.getMessage());
        return ERROR_TEMPLATE;
    }

    @ExceptionHandler(MethodArgumentTypeMismatchException.class)
    public String handleNotValidArgument() {
        return "redirect:/customer/all";
    }

    @ExceptionHandler(InvalidUserRole.class)
    public String invalidUserRole(Model model) {
        model.addAttribute(MESSAGE, "Nesprávná role uživatele");
        return ERROR_TEMPLATE;
    }

    @ExceptionHandler(UsernameNotFoundException.class)
    public String handleInvalidUserName(Model model) {
        model.addAttribute(MESSAGE, "Nesprávné uživatelské jméno nebo heslo");
        return ERROR_TEMPLATE;
    }

    @ExceptionHandler(InvalidPasswordChangeException.class)
    public String handleInvalidPassword(Model model, RedirectAttributes redirectAttributes) {
        redirectAttributes.addFlashAttribute(FAILURE_MESSAGE, "Nesprávné současné heslo, nebo se nová hesla neshodují.");
        return USER_PASSWORD_TEMPLATE;
    }
}
