package cz.simecek.inspectionmanager.model;

import org.springframework.data.annotation.CreatedDate;

import javax.persistence.*;
import javax.validation.constraints.Email;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.sql.Date;
import java.util.Set;

@Entity
public class Customer {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    @OneToMany(mappedBy = "customer")
    private Set<Machine> machines;

    @OneToMany(mappedBy = "customer")
    private Set<CustomerHistory> customerHistories;

    @Size(max = 255)
    private String firstName;

    @Size(max = 255)
    @NotBlank
    private String lastName;

    @Size(max = 255)
    private String city;

    @Size(max = 9)
    private String phone;

    @Email
    private String email;

    @Size(max = 255)
    private String focus;

    @CreatedDate
    private Date createdAt;

    @Column
    private Boolean isDeleted;

    @ManyToOne
    @JoinColumn(nullable = false)
    private SystemUser createdBy;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public String getCity() {
        return city;
    }

    public void setCity(String city) {
        this.city = city;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getFocus() {
        return focus;
    }

    public void setFocus(String focus) {
        this.focus = focus;
    }

    public Date getCreatedAt() {
        return createdAt;
    }

    public void setCreatedAt(Date createdAt) {
        this.createdAt = createdAt;
    }

    public Set<Machine> getMachines() {
        return machines;
    }

    public void setMachines(Set<Machine> machines) {
        this.machines = machines;
    }

    public Set<CustomerHistory> getCustomerHistories() {
        return customerHistories;
    }

    public void setCustomerHistories(Set<CustomerHistory> customerHistories) {
        this.customerHistories = customerHistories;
    }

    public Boolean getDeleted() {
        return isDeleted;
    }

    public void setDeleted(Boolean deleted) {
        isDeleted = deleted;
    }

    public SystemUser getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(SystemUser createdBy) {
        this.createdBy = createdBy;
    }

    @Override
    public String toString() {
        return "Zákazník {" +
                " jméno='" + firstName + '\'' +
                ", příjmení='" + lastName + '\'' +
                ", město='" + city + '\'' +
                ", telefon='" + phone + '\'' +
                ", email='" + email + '\'' +
                ", poznámka='" + focus + '\'' +
                ", vytvořen=" + createdAt +
                ", smazán=" + isDeleted +
                '}';
    }
}
