package cz.simecek.inspectionmanager.model;

import javax.persistence.*;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.sql.Date;
import java.util.HashSet;
import java.util.Set;

@Entity
public class Machine {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    @ManyToOne
    @JoinColumn(nullable = false)
    private Customer customer;

    @OneToMany(mappedBy = "machine")
    private Set<Inspection> inspections;

    @ManyToMany(fetch = FetchType.LAZY,
            cascade = {CascadeType.PERSIST,
                    CascadeType.MERGE})
    @JoinTable(name = "category_machine",
            joinColumns = {@JoinColumn(name = "machine_id")},
            inverseJoinColumns = {@JoinColumn(name = "category_id")})
    private Set<Category> categories = new HashSet<>();

    @ManyToMany(fetch = FetchType.LAZY,
            cascade = {CascadeType.PERSIST,
                    CascadeType.MERGE})
    @JoinTable(name = "sparePart_machine",
            joinColumns = {@JoinColumn(name = "machine_id")},
            inverseJoinColumns = {@JoinColumn(name = "sparePart_id")})
    private Set<SparePart> spareParts = new HashSet<>();

    @OneToMany(mappedBy = "machine")
    private Set<MachineHistory> machineHistories;

    @NotBlank
    @Size(max = 255)
    private String name;

    @NotBlank
    @Size(max = 255)
    @Column(unique = true)
    private String code;

    @Column
    private Date nextInspectionAt;

    @Size(max = 255)
    private String note;

    @Column
    private Boolean isBoughtLocally;

    @Column
    private Date boughtAt;

    @Column
    private Date createdAt;

    @Column
    private Boolean isWorking;

    @Column
    private Boolean isDeleted;

    @ManyToOne
    @JoinColumn(nullable = false)
    private SystemUser createdBy;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Customer getCustomer() {
        return customer;
    }

    public void setCustomer(Customer customer) {
        this.customer = customer;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public Date getNextInspectionAt() {
        return nextInspectionAt;
    }

    public void setNextInspectionAt(Date nextInspectionAt) {
        this.nextInspectionAt = nextInspectionAt;
    }

    public String getNote() {
        return note;
    }

    public void setNote(String note) {
        this.note = note;
    }

    public Boolean getIsBoughtLocally() {
        return isBoughtLocally;
    }

    public void setIsBoughtLocally(Boolean boughtLocally) {
        isBoughtLocally = boughtLocally;
    }

    public Date getBoughtAt() {
        return boughtAt;
    }

    public void setBoughtAt(Date boughtAt) {
        this.boughtAt = boughtAt;
    }

    public Date getCreatedAt() {
        return createdAt;
    }

    public void setCreatedAt(Date createdAt) {
        this.createdAt = createdAt;
    }

    public Boolean getIsWorking() {
        return isWorking;
    }

    public void setIsWorking(Boolean working) {
        isWorking = working;
    }


    public Set<Inspection> getInspections() {
        return inspections;
    }

    public void setInspections(Set<Inspection> inspections) {
        this.inspections = inspections;
    }

    public Boolean getBoughtLocally() {
        return isBoughtLocally;
    }

    public void setBoughtLocally(Boolean boughtLocally) {
        isBoughtLocally = boughtLocally;
    }

    public Boolean getWorking() {
        return isWorking;
    }

    public void setWorking(Boolean working) {
        isWorking = working;
    }

    public Boolean getDeleted() {
        return isDeleted;
    }

    public void setDeleted(Boolean deleted) {
        isDeleted = deleted;
    }

    public Set<Category> getCategories() {
        return categories;
    }

    public void setCategories(Set<Category> categories) {
        this.categories = categories;
    }

    public SystemUser getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(SystemUser createdBy) {
        this.createdBy = createdBy;
    }

    public Set<SparePart> getSpareParts() {
        return spareParts;
    }

    public void setSpareParts(Set<SparePart> spareParts) {
        this.spareParts = spareParts;
    }

    public Set<MachineHistory> getMachineHistories() {
        return machineHistories;
    }

    public void setMachineHistories(Set<MachineHistory> machineHistories) {
        this.machineHistories = machineHistories;
    }

    @Override
    public String toString() {
        return "Stroj {" +
                " jméno='" + name + '\'' +
                ", kód='" + code + '\'' +
                ", dalšíKontrola=" + nextInspectionAt +
                ", poznámka='" + note + '\'' +
                ", koupenoLokálně=" + isBoughtLocally +
                ", koupeno=" + boughtAt +
                ", vytvořeno=" + createdAt +
                ", funguje=" + isWorking +
                ", smazáno=" + isDeleted +
                '}';
    }
}
