package cz.simecek.inspectionmanager.service.user_detail;

import cz.simecek.inspectionmanager.model.SystemUser;
import cz.simecek.inspectionmanager.repository.SystemUserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Set;

@Service
public class UserDetailsServiceImpl implements UserDetailsService {

    private final SystemUserRepository userRepository;

    @Autowired
    public UserDetailsServiceImpl(SystemUserRepository userRepository) {
        this.userRepository = userRepository;
    }

    public UserDetails loadUserByUsername(String userName) {
        SystemUser user = userRepository.findByUserName(userName);

        if (user == null || user.getBlocked()) {
            throw new UsernameNotFoundException(userName);
        }
        Set<GrantedAuthority> grantedAuthoritySet = new HashSet<>();
        grantedAuthoritySet.add(new SimpleGrantedAuthority(user.getRole().getName()));
        return new org.springframework.security.core.userdetails.User(user.getUserName(), user.getPassword(), grantedAuthoritySet);
    }
}
